# Copyright 2020, Red Hat, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Generate a listing of all known traits."""

import os

import os_traits

REFERENCE_DIR = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), os.pardir, 'source',
    'reference',
)


def _generate_trait_list():
    yield 'Available Traits'
    yield '================'
    yield ''
    yield 'Below is a list of all traits currently available.'
    yield ''

    for trait in os_traits.get_traits():
        yield f'- ``{trait}``'

    yield ''


def generate_trait_list(app):
    """Generate a listing of all known traits."""
    with open(os.path.join(REFERENCE_DIR, 'traits.rst'), 'w+') as fh:
        for line in _generate_trait_list():
            print(line, file=fh)


def setup(app):
    app.connect('builder-inited', generate_trait_list)
    return {
        'parallel_read_safe': True,
        'parallel_write_safe': True,
    }



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

TRAITS = [
    # traits corresponding to the allowed values of "hw_architecture"
    # image metadata property
    # https://github.com/openstack/nova/blob/1f74441/nova/objects/fields.py#L120-228
    'ALPHA',
    'ARMV6',
    'ARMV7',
    'ARMV7B',
    'AARCH64',
    'CRIS',
    'I686',
    'IA64',
    'LM32',
    'M68K',
    'MICROBLAZE',
    'MICROBLAZEEL',
    'MIPS',
    'MIPSEL',
    'MIPS64',
    'MIPS64EL',
    'OPENRISC',
    'PARISC',
    'PARISC64',
    'PPC',
    'PPCLE',
    'PPC64',
    'PPC64LE',
    'PPCEMB',
    'S390',
    'S390X',
    'SH4',
    'SH4EB',
    'SPARC',
    'SPARC64',
    'UNICORE32',
    'X86_64',
    'XTENSA',
    'XTENSAEB',
]



# -*- coding: utf-8 -*-
# Copyright 2017 Arm Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


TRAITS = [
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=9703d9d7f
    'FP',
    'ASIMD',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=46efe547a
    'EVTSTRM',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=4bff28ccd
    'AES',
    'PMULL',
    'SHA1',
    'SHA2',
    'CRC32',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=bf5006184
    'FPHP',
    'ASIMDHP',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=f92f5ce01
    'ASIMDRDM',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=40a1db243
    'ATOMICS',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=c8c3798d2
    'JSCVT',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=cb567e79f
    'FCMA',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=c651aae5a
    'LRCPC',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=7aac405eb
    'DCPOP',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=f5e035f86
    'SHA3',
    'SM3',
    'SM4',
    'ASIMDDP',
    'SHA512',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=43994d824
    'SVE',
    # ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/
    # linux.git/commit/?id=77c97b4ee
    'CPUID',
]



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from os_traits.hw.gpu import cuda
from os_traits.tests import base


class TestCUDA(base.TestCase):

    def test_unknown_sdk_support(self):
        self.assertIsNone(cuda.compute_capabilities_supported('UNKNOWN'))

    def test_sdk6_5_support(self):
        expected = set([
            cuda.COMPUTE_CAPABILITY_V1_0,
            cuda.COMPUTE_CAPABILITY_V1_1,
            cuda.COMPUTE_CAPABILITY_V1_2,
            cuda.COMPUTE_CAPABILITY_V1_3,
            cuda.COMPUTE_CAPABILITY_V2_0,
            cuda.COMPUTE_CAPABILITY_V2_1,
            cuda.COMPUTE_CAPABILITY_V3_0,
            cuda.COMPUTE_CAPABILITY_V3_2,
            cuda.COMPUTE_CAPABILITY_V3_5,
            cuda.COMPUTE_CAPABILITY_V3_7,
            cuda.COMPUTE_CAPABILITY_V5_0,
            cuda.COMPUTE_CAPABILITY_V5_2,
            cuda.COMPUTE_CAPABILITY_V5_3,
        ])

        actual = cuda.compute_capabilities_supported(cuda.SDK_V6_5)
        self.assertEqual(expected, actual)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


TRAITS = [
    # Required as mitigation for "MDS" (Microarchitectural Data
    # Sampling) security flaws
    'MD_CLEAR',
    # ref: https://git.qemu.org/?p=qemu.git;a=blob;f=docs/qemu-cpu-models.texi
    # (Important CPU features for Intel x86 hosts)
    'PCID',
    'SPEC_CTRL',
    'SSBD',
    # ref: https://en.wikipedia.org/wiki/VT-x
    'VMX',
]



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

TRAITS = [
    'TSO',  # TCP segmentation
    'GRO',  # Generic receive
    'GSO',  # Generic segmentation
    'UFO',  # UDP Fragmentation
    'LRO',  # Large receive
    'LSO',  # Large send
    'TCS',  # TCP Checksum
    'UCS',  # UDP Checksum
    'SCS',  # SCTP Checksum
    'L2CRC',  # Layer-2 CRC
    'FDF',  # Intel Flow-Director Filter
    'RXVLAN',  # VLAN receive tunnel segmentation
    'TXVLAN',  # VLAN transmit tunnel segmentation
    'VXLAN',  # VxLAN tunneling
    'GRE',  # GRE tunneling
    'GENEVE',  # Geneve tunneling
    'TXUDP',  # UDP transmit tunnel segmentation
    'QINQ',  # QinQ specification
    'RDMA',  # remote direct memory access
    'RXHASH',  # receive hashing
    'RX',  # RX checksumming
    'TX',  # RX checksumming
    'SG',  # scatter-gather
    'SWITCHDEV',  # Offload datapath rules
]



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

TRAITS = [
    # The virt driver supports attaching a network interface after boot
    'ATTACH_INTERFACE',
    # The virt driver supports attaching a network interface after boot and
    # specifying a device tag for the interface
    'ATTACH_INTERFACE_WITH_TAG',
    # The virt driver supports Packed virtqueue format
    'VIRTIO_PACKED',
    # traits corresponding to the allowed values of "hw_vif_model"
    # image metadata property
    # https://github.com/openstack/nova/blob/1f74441/nova/network/model.py#L136-L149
    'VIF_MODEL_E1000',
    'VIF_MODEL_E1000E',
    'VIF_MODEL_LAN9118',
    'VIF_MODEL_NETFRONT',
    'VIF_MODEL_NE2K_PCI',
    'VIF_MODEL_PCNET',
    'VIF_MODEL_RTL8139',
    'VIF_MODEL_SPAPR_VLAN',
    'VIF_MODEL_SRIOV',
    'VIF_MODEL_VIRTIO',
    'VIF_MODEL_VMXNET',
    'VIF_MODEL_VMXNET3',
    'VIF_MODEL_IGB',
]



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# A few generalized capabilities of some RP
TRAITS = [
    # The resource provider decorated with this trait indicates that
    # hyperthreading is enabled on the provider.
    # Operators of resource-constrained systems would be able to decorate a
    # single NUMA node resource provider on a multi-socket system with this
    # HW_CPU_HYPERTHREADING trait to “carve out” a part of the host system
    # for guests that can tolerate hyperthread siblings providing CPU
    # resources.
    'HYPERTHREADING',
]



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

TRAITS = [
    # traits corresponding to the allowed values of "hw_disk_bus"
    # and "hw_cdrom_bus" image metadata properties
    # https://github.com/openstack/nova/blob/1f74441/nova/objects/fields.py#L320-L332
    'BUS_FDC',
    'BUS_IDE',
    'BUS_LXC',
    'BUS_SATA',
    'BUS_SCSI',
    'BUS_USB',
    'BUS_VIRTIO',
    'BUS_UML',
    'BUS_XEN',
    # The compute manager supports virtio filesystems.
    'VIRTIO_FS',
]
